#property indicator_chart_window

//------- Input parameters of the indicator -------------------------------
extern int    NumberOfDays = 5;         // Number of days
extern string AsiaBegin    = "01:00";   // Open time of the Asian session
extern string AsiaEnd      = "10:00";   // Close time of the Asian session
extern color  AsiaColor    = Red;       // Color of the Asian session
extern string EurBegin     = "07:00";   // Open time of the European session
extern string EurEnd       = "16:00";   // Close time of the European session
extern color  EurColor     = Tan;       // Color of the European session
extern string USABegin     = "14:00";   // Open time of the American session
extern string USAEnd       = "23:00";   // Close time of the American session
extern color  USAColor     = Pink;      // Color of the American session
extern bool   ShowPrice    = True;      // Show price levels
extern color  clFont       = Blue;      // Font color
extern int    SizeFont     = 8;         // Font size
extern int    OffSet       = 10;        // Offset
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void init() 
  {
   DeleteObjects();
   for(int i=0;i<NumberOfDays;i++) 
     {
      CreateObjects("AS"+i,AsiaColor);
      CreateObjects("EU"+i,EurColor);
      CreateObjects("US"+i,USAColor);
     }
   Comment("");
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void deinit() 
  {
   DeleteObjects();
   Comment("");
  }
//+------------------------------------------------------------------+
//| Indicator's objects creation                                     |
//| Parameters:                                                      |
//|   no - Object name                                               |
//|   cl - Object color                                              |
//+------------------------------------------------------------------+
void CreateObjects(string no,color cl) 
  {
   ObjectCreate(no,OBJ_RECTANGLE,0,0,0,0,0);
   ObjectSet(no,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(no,OBJPROP_COLOR,cl);
   ObjectSet(no,OBJPROP_BACK,True);
  }
//+------------------------------------------------------------------+
//| Indicator's object deletion                                      |
//+------------------------------------------------------------------+
void DeleteObjects() 
  {
   for(int i=0;i<NumberOfDays;i++) 
     {
      ObjectDelete("AS"+i);
      ObjectDelete("EU"+i);
      ObjectDelete("US"+i);
     }
   ObjectDelete("ASup");
   ObjectDelete("ASdn");
   ObjectDelete("EUup");
   ObjectDelete("EUdn");
   ObjectDelete("USup");
   ObjectDelete("USdn");
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
void start() 
  {
   datetime dt=CurTime();

   for(int i=0;i<NumberOfDays;i++) 
     {
      if(ShowPrice && i==0) 
        {
         DrawPrices(dt,"AS",AsiaBegin,AsiaEnd);
         DrawPrices(dt,"EU",EurBegin,EurEnd);
         DrawPrices(dt,"US",USABegin,USAEnd);
        }
      DrawObjects(dt,"AS"+i,AsiaBegin,AsiaEnd);
      DrawObjects(dt,"EU"+i,EurBegin,EurEnd);
      DrawObjects(dt,"US"+i,USABegin,USAEnd);
      dt=decDateTradeDay(dt);
      while(TimeDayOfWeek(dt)<1 || TimeDayOfWeek(dt)>5) dt=decDateTradeDay(dt);
     }
  }
//+------------------------------------------------------------------+
//| Indicator's object drawing on the chart                          |
//| Parameters:                                                      |
//|   dt - Date of the trading day                                   |
//|   no - Object name                                               |
//|   tb - Time of session begin                                     |
//|   te - Time of session end                                       |
//+------------------------------------------------------------------+
void DrawObjects(datetime dt,string no,string tb,string te) 
  {
   datetime t1,t2;
   double   p1,p2;
   int      b1,b2;

   t1=StrToTime(TimeToStr(dt, TIME_DATE)+" "+tb);
   t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
   b1=iBarShift(NULL, 0, t1);
   b2=iBarShift(NULL, 0, t2);
   p1=High[Highest(NULL, 0, MODE_HIGH, b1-b2+1, b2)];
   p2=Low [Lowest (NULL, 0, MODE_LOW , b1-b2+1, b2)];
   ObjectSet(no,OBJPROP_TIME1,t1);
   ObjectSet(no,OBJPROP_PRICE1,p1);
   ObjectSet(no,OBJPROP_TIME2,t2);
   ObjectSet(no,OBJPROP_PRICE2,p2);
  }
//+------------------------------------------------------------------+
//| Drawing of price labels on the chart                             |
//| Parameters:                                                      |
//|   dt - Date of the trading day                                   |
//|   no - Object name                                               |
//|   tb - Time of session begin                                     |
//|   te - Time of session end                                       |
//+------------------------------------------------------------------+
void DrawPrices(datetime dt,string no,string tb,string te) 
  {
   datetime t1,t2;
   double   p1,p2;
   int      b1,b2;

   t1=StrToTime(TimeToStr(dt, TIME_DATE)+" "+tb);
   t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
   b1=iBarShift(NULL, 0, t1);
   b2=iBarShift(NULL, 0, t2);
   p1=High[Highest(NULL, 0, MODE_HIGH, b1-b2, b2)];
   p2=Low [Lowest (NULL, 0, MODE_LOW , b1-b2, b2)];

   if(ObjectFind(no+"up")<0) ObjectCreate(no+"up",OBJ_TEXT,0,0,0);
   ObjectSet(no+"up",OBJPROP_TIME1,t2);
   ObjectSet(no+"up",OBJPROP_PRICE1,p1+(OffSet+SizeFont)*Point);
   ObjectSet(no+"up",OBJPROP_COLOR,clFont);
   ObjectSet(no+"up",OBJPROP_FONTSIZE,SizeFont);
   ObjectSetText(no+"up",DoubleToStr(p1+Ask-Bid,Digits));

   if(ObjectFind(no+"dn")<0) ObjectCreate(no+"dn",OBJ_TEXT,0,0,0);
   ObjectSet(no+"dn",OBJPROP_TIME1,t2);
   ObjectSet(no+"dn",OBJPROP_PRICE1,p2-OffSet*Point);
   ObjectSet(no+"dn",OBJPROP_COLOR,clFont);
   ObjectSet(no+"dn",OBJPROP_FONTSIZE,SizeFont);
   ObjectSetText(no+"dn",DoubleToStr(p2,Digits));
  }
//+------------------------------------------------------------------+
//| Decreasing the date by one trading day                           |
//| Parameters:                                                      |
//|   dt - Date of the trading day                                   |
//+------------------------------------------------------------------+
datetime decDateTradeDay(datetime dt) 
  {
   int ty=TimeYear(dt);
   int tm=TimeMonth(dt);
   int td=TimeDay(dt);
   int th=TimeHour(dt);
   int ti=TimeMinute(dt);

   td--;
   if(td==0) 
     {
      tm--;
      if(tm==0) 
        {
         ty--;
         tm=12;
        }
      if(tm==1 || tm==3 || tm==5 || tm==7 || tm==8 || tm==10 || tm==12) td=31;
      if(tm==2) if(MathMod(ty, 4)==0) td=29; else td=28;
      if(tm==4 || tm==6 || tm==9 || tm==11) td=30;
     }
   return(StrToTime(ty+"."+tm+"."+td+" "+th+":"+ti));
  }
//+------------------------------------------------------------------+

